#!/usr/bin/php -q
<?php
/* vim: set noexpandtab tabstop=2 softtabstop=2 shiftwidth=2: */

/**
 * extractgbxdata.php - Extract & show GBX challenge/map/replay/pack data for
 *                      TrackMania (TM) & ManiaPlanet (MP) files
 * Created by Xymph <tm@gamers.org>
 *
 * v2.3: Add Vehicle field for maps & replays; add HdrSize & Download fields and
 *       list of Included Pack headers for packs
 * v2.2: Add GhostBks field for maps; add IncPacks field for packs
 * v2.1: Add Author chunk fields to output; add Pack support; enhance output layout
 * v2.0: Complete rewrite
 */

	require_once('gbxdatafetcher.inc.php');

	function stripColors($str) {
		return
			str_replace("\0", '$$',
				preg_replace('/\\$[hlp](.*?)(?:\\[.*?\\](.*?))*(?:\\$[hlp]|$)/iu', '$1$2',
					preg_replace('/\\$(?:[0-9a-f][^$][^$]|[0-9a-f][^$]|[^][hlp]|(?=[][])|$)/iu', '',
						str_replace('$$', "\0", $str)
					)
				)
			)
		;
	}

	function stripNewlines($str) {
		return
			str_replace(array("\n\n", "\r", "\n"),
			            array(' ', '', ''), $str);
	}

	function formatTime($MwTime, $msec) {
		if ($MwTime == -1) {
			return '???';
		} else {
			$minutes = floor($MwTime/(1000*60));
			$seconds = floor(($MwTime - $minutes*60*1000)/1000);
			// include thousandths?
			if ($msec) {
				$mseconds = substr($MwTime, strlen($MwTime)-3);
				$tm = sprintf('%02d:%02d.%03d', $minutes, $seconds, $mseconds);
			} else { // include hundredths
				$hseconds = substr($MwTime, strlen($MwTime)-3, 2);
				$tm = sprintf('%02d:%02d.%02d', $minutes, $seconds, $hseconds);
			}
		}
		if ($tm[0] == '0') {
			$tm = substr($tm, 1);
		}
		return $tm;
	}


	// check for -tn flag
	$writetn = false;
	if (isset($argv[1]) && $argv[1] == '-tn') {
		$writetn = true;
		array_shift($argv);
	}
	$tryagain = false;

	// check for file name
	if (!isset($argv[1]) || $argv[1] == '') {
		echo $argv[0] . ": Missing GBX filename\n";
		return;
	}
	$filename = $argv[1];


	// check for challenge/map and show info
	$gbx = new GBXChallMapFetcher(true, true);
	try
	{
		$gbx->processFile($filename);

		echo 'UId     : ' . $gbx->uid . "\n";
		echo 'HdrVersn: ' . $gbx->headerVersn . "\n";
		echo 'Name    : ' . $gbx->name . "\n";
		echo '   strip: ' . trim(stripColors(stripNewlines($gbx->name))) . "\n";
		echo 'AuthLogn: ' . $gbx->author . "\n";
		if ($gbx->authorNick != '') {
			echo 'AuthNick: ' . $gbx->authorNick . "\n";
			echo '   strip: ' . trim(stripColors($gbx->authorNick)) . "\n";
		}
		if ($gbx->authorZone != '')
			echo 'AuthZone: ' . $gbx->authorZone . "\n";
		if ($gbx->authorEInfo != '')
			echo 'AuthInfo: ' . $gbx->authorEInfo . "\n";
		echo 'Type    : ' . $gbx->type . '  ' . $gbx->typeName . "\n";
		echo 'Kind    : ' . $gbx->kind . '  ' . $gbx->kindName . "\n";
		if ($gbx->mapType != '' || $gbx->mapStyle != '') {
			echo 'MapType : ' . $gbx->mapType . "\n";
			echo 'MapStyle: ' . $gbx->mapStyle . "\n";
		}

		echo 'Multilap: ' . ($gbx->multiLap ? 'Yes' : 'No') . "\n";
		echo 'Editor  : ' . ($gbx->simpleEdit ? 'Simple' : 'Advanced') . "\n";
		echo 'GhostBks: ' . ($gbx->ghostBlocks ? 'Yes' : 'No') . "\n";
		echo 'NbChecks: ' . $gbx->nbChecks . "\n";
		echo 'NbLaps  : ' . $gbx->nbLaps . "\n";
		echo 'Lightmap: ' . $gbx->lightmap . "\n";

		echo 'Environ.: ' . $gbx->envir . "\n";
		echo 'Mood    : ' . $gbx->mood . "\n";
		echo 'Envir.BG: ' . $gbx->envirBg . "\n";
		echo 'AuthorBG: ' . $gbx->authorBg . "\n";
		if ($gbx->titleUid != '')
			echo 'TitleUid: ' . $gbx->titleUid . "\n";
		if ($gbx->vehicle != '')
			echo 'Vehicle : ' . $gbx->vehicle . "\n";

		// check for millisecs in MP engine
		$msec = ($gbx->exeVer != '' && $gbx->exeVer[0] >= 3);
		if ($gbx->type != 6)  // Script
			echo 'AuthorTm: ' . sprintf("%6d  %9s\n", $gbx->authorTime, formatTime($gbx->authorTime, $msec));

		if ($gbx->type == 1) {  // Platform
			echo 'Gold  CP: ' . $gbx->goldTime . "\n";
			echo 'SilverCP: ' . $gbx->silverTime . "\n";
			echo 'BronzeCP: ' . $gbx->bronzeTime . "\n";
		} elseif ($gbx->type == 5) {  // Stunts
			echo 'Gold  Sc: ' . $gbx->goldTime . "\n";
			echo 'SilverSc: ' . $gbx->silverTime . "\n";
			echo 'BronzeSc: ' . $gbx->bronzeTime . "\n";
		} elseif ($gbx->type == 6) {  // Script
			echo 'AuthorTm: ' . $gbx->authorTime . "\n";
			echo 'Gold  Tm: ' . $gbx->goldTime . "\n";
			echo 'SilverTm: ' . $gbx->silverTime . "\n";
			echo 'BronzeTm: ' . $gbx->bronzeTime . "\n";
		} else {
			echo 'Gold  Tm: ' . sprintf("%6d  %9s\n", $gbx->goldTime, formatTime($gbx->goldTime, $msec));
			echo 'SilverTm: ' . sprintf("%6d  %9s\n", $gbx->silverTime, formatTime($gbx->silverTime, $msec));
			echo 'BronzeTm: ' . sprintf("%6d  %9s\n", $gbx->bronzeTime, formatTime($gbx->bronzeTime, $msec));
		}

		echo 'Cost    : ' . $gbx->cost . "\n";
		echo 'AuthScor: ' . $gbx->authorScore . "\n";
		echo 'Password: ';
		// convert password to hex format
		if (($p = $gbx->password) != '') {
			for ($i = 3; $i < strlen($p); $i++) {  // skip 3 bogus chars
				printf('%02X', ord($p{$i}));
			}
		}
		echo "\n";

		if ($gbx->headerVersn > 5) {
			echo 'Thumblen: ' . $gbx->thumbLen . "\n";
			echo 'Comment : ' . $gbx->comment . "\n";
		}
		echo "\n";
		echo "XML     :\n" . str_replace("><", ">\n<", $gbx->xml) . "\n";

		// write thumbnail to file?
		if ($writetn) {
			$tnname = preg_replace('/\.Gbx$/i', '.jpg', $filename);
			if (@file_put_contents($tnname, $gbx->thumbnail) === false)
				echo "$tnfile: Cannot write thumbnail file\n";
		}
	}
	catch (Exception $e)
	{
		// check for non-ChallMap types/classes?
		if ($e->getCode() == 5 || $e->getCode() == 7)
			$tryagain = true;
		else
			echo $e->getMessage() . "\n";
	}

	if (!$tryagain) return;
	$tryagain = false;


	// check for replay and show info
	$gbx = new GBXReplayFetcher(true);
	try
	{
		$gbx->processFile($filename);

		echo 'UId     : ' . $gbx->uid . "\n";
		echo 'Environ.: ' . $gbx->envir . "\n";
		echo 'AuthLogn: ' . $gbx->author . "\n";
		if ($gbx->authorNick != '') {
			echo 'AuthNick: ' . $gbx->authorNick . "\n";
			echo '   strip: ' . trim(stripColors($gbx->authorNick)) . "\n";
		}
		if ($gbx->authorZone != '')
			echo 'AuthZone: ' . $gbx->authorZone . "\n";
		if ($gbx->authorEInfo != '')
			echo 'AuthInfo: ' . $gbx->authorEInfo . "\n";
		if ($gbx->login != '')
			echo 'DrvLogin: ' . $gbx->login . "\n";
		echo 'DrvNickn: ' . $gbx->nickname . "\n";
		echo '   strip: ' . trim(stripColors($gbx->nickname)) . "\n";
		if ($gbx->vehicle != '')
			echo 'Vehicle : ' . $gbx->vehicle . "\n";

		// check for millisecs in MP engine
		$msec = ($gbx->exeVer != '' && $gbx->exeVer[0] >= 3);
		echo 'Replay  : ' . sprintf("%6d  %9s\n", $gbx->replay, formatTime($gbx->replay, $msec));

		echo "\n";
		echo "XML     :\n" . str_replace("><", ">\n<", $gbx->xml) . "\n";
	}
	catch (Exception $e)
	{
		// check for non-Replay types/classes?
		if ($e->getCode() == 5 || $e->getCode() == 7)
			$tryagain = true;
		else
			echo $e->getMessage() . "\n";
	}

	if (!$tryagain) return;


	// check for pack and show info
	$gbx = new GBXPackFetcher(true);
	try
	{
		$gbx->processFile($filename);

		echo 'HdrVersn: ' . $gbx->headerVersn . "\n";
		echo 'Flags   : ' . sprintf("%08X", $gbx->flags) . "\n";
		if ($gbx->headerMaxSz > 0) {
			switch ($gbx->headerMaxSz) {
				case 0x0004000: $headerLabel = 'small'; break;
				case 0x0100000: $headerLabel = 'big'; break;
				case 0x1000000: $headerLabel = 'huge'; break;
				default       : $headerLabel = 'unknown'; break;
			}
			echo 'HdrSize : ' . sprintf("%08X", $gbx->headerMaxSz) . " ($headerLabel)\n";
		}
		echo 'UsageDir: ' . $gbx->usageSubdir . "\n";
		echo 'Manialnk: ' . $gbx->infoMlUrl . "\n";
		echo 'Download: ' . $gbx->downloadUrl . "\n";
		if ($gbx->creatDate != -1)
			echo 'Creation: ' . gmdate('Y-M-d H:i:s', $gbx->creatDate) . " UTC\n";
		else
			echo 'Creation: 0' . "\n";
		echo 'BldInfo : ' . $gbx->buildInfo . "\n";
		echo 'AuthUrl : ' . $gbx->authorUrl . "\n";
		if ($gbx->titleId != '')
			echo 'TitleId : ' . $gbx->titleId . "\n";
		echo 'AuthLogn: ' . $gbx->authorLogin . "\n";
		if ($gbx->authorNick != '') {
			echo 'AuthNick: ' . $gbx->authorNick . "\n";
			echo '   strip: ' . trim(stripColors($gbx->authorNick)) . "\n";
		}
		if ($gbx->authorZone != '')
			echo 'AuthZone: ' . $gbx->authorZone . "\n";
		if ($gbx->authorEInfo != '')
			echo 'AuthInfo: ' . $gbx->authorEInfo . "\n";
		echo 'Comment : ' . rtrim($gbx->comment) . "\n";
		echo 'IncPacks: ' . $gbx->nbPacks . "\n";

		echo "\n";
		echo "XML     :\n" . str_replace("><", ">\n<", $gbx->xml) . "\n";

		for ($i = 0; $i < $gbx->nbPacks; $i++) {
			$hdr = $gbx->packHeaders[$i];
			echo "\n";
			echo 'Included Pack ' . ($i+1) . "\n";
			echo 'Name    : ' . $hdr->name . "\n";
			echo 'Manialnk: ' . $hdr->infoMlUrl . "\n";
			if ($hdr->creatDate != -1)
				echo 'Creation: ' . gmdate('Y-M-d H:i:s', $hdr->creatDate) . " UTC\n";
			else
				echo 'Creation: 0' . "\n";
			echo 'NestgLvl: ' . $hdr->nestLevel . "\n";
			echo 'AuthLogn: ' . $hdr->authorLogin . "\n";
			if ($hdr->authorNick != '') {
				echo 'AuthNick: ' . $hdr->authorNick . "\n";
				echo '   strip: ' . trim(stripColors($hdr->authorNick)) . "\n";
			}
			if ($hdr->authorZone != '')
				echo 'AuthZone: ' . $hdr->authorZone . "\n";
			if ($hdr->authorEInfo != '')
				echo 'AuthInfo: ' . $hdr->authorEInfo . "\n";
		}
	}
	catch (Exception $e)
	{
		if ($e->getCode() == 5 || $e->getCode() == 7)
			echo $argv[0] . ": Unsupported GBX type/class\n";
		else
			echo $e->getMessage() . "\n";
	}
?>
